package timing.ukulele.auth.config;

import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.oauth2.jwt.JwtDecoder;
import org.springframework.security.oauth2.jwt.JwtEncoder;
import org.springframework.security.web.context.HttpSessionSecurityContextRepository;
import org.springframework.security.web.context.SecurityContextRepository;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;
import timing.ukulele.auth.config.property.TimingSecurityProperties;
import timing.ukulele.auth.security.filter.JWTSecurityFilter;
import timing.ukulele.auth.security.handler.LoginFailureHandler;
import timing.ukulele.auth.security.handler.LoginSuccessHandler;
import timing.ukulele.auth.security.service.TimingUserDetailService;
import timing.ukulele.auth.security.type.image.ImageCodeAuthenticationSecurityConfig;
import timing.ukulele.auth.security.type.sms.SmsCodeAuthenticationSecurityConfig;
import timing.ukulele.auth.security.type.wechat.ThirdQrAuthenticationSecurityConfig;

/**
 * 将bean注入至ioc的配置类
 */
@Configuration
@EnableConfigurationProperties
public class BeanConfig {

    public BeanConfig() {
//        this.securityContextRepository = securityContextRepository;
    }

    /**
     * 跨域过滤器配置
     *
     * @return CorsFilter
     */
    @Bean
    public CorsFilter corsFilter() {

        // 初始化cors配置对象
        CorsConfiguration configuration = new CorsConfiguration();

        // 设置允许跨域的域名,如果允许携带cookie的话,路径就不能写*号, *表示所有的域名都可以跨域访问
        configuration.addAllowedOrigin("*");
        // 设置跨域访问可以携带cookie
        configuration.setAllowCredentials(false);
        // 允许所有的请求方法 ==> GET POST PUT Delete
        configuration.addAllowedMethod("*");
        // 允许携带任何头信息
        configuration.addAllowedHeader("*");

        // 初始化cors配置源对象
        UrlBasedCorsConfigurationSource configurationSource = new UrlBasedCorsConfigurationSource();

        // 给配置源对象设置过滤的参数
        // 参数一: 过滤的路径 == > 所有的路径都要求校验是否跨域
        // 参数二: 配置类
        configurationSource.registerCorsConfiguration("/**", configuration);

        // 返回配置好的过滤器
        return new CorsFilter(configurationSource);
    }

    @Bean
    public JWTSecurityFilter jwtAuthenticationFilter(JwtDecoder jwtDecoder) {
        return new JWTSecurityFilter(jwtDecoder);
    }

    @Bean
    public LoginSuccessHandler loginSuccessHandler(JwtEncoder jwtEncoder,
                                                   TimingSecurityProperties securityProperties) {
        return new LoginSuccessHandler(jwtEncoder, securityProperties);
    }

    @Bean
    public LoginFailureHandler loginFailureHandler() {
        return new LoginFailureHandler();
    }

    @Bean
    public PasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }

    @Bean
    public TimingUserDetailService userDetailService(PasswordEncoder passwordEncoder) {
        return new TimingUserDetailService(passwordEncoder);
    }

    @Bean
    public SmsCodeAuthenticationSecurityConfig smsCodeAuthenticationSecurityConfig(
            LoginSuccessHandler loginSuccessHandler,
            LoginFailureHandler loginFailureHandler,
            TimingUserDetailService userDetailsService) {
        return new SmsCodeAuthenticationSecurityConfig(loginFailureHandler, loginSuccessHandler, userDetailsService);
    }

    @Bean
    public ImageCodeAuthenticationSecurityConfig imageCodeAuthenticationSecurityConfig(
            LoginSuccessHandler loginSuccessHandler,
            LoginFailureHandler loginFailureHandler,
            TimingUserDetailService userDetailsService,
            PasswordEncoder passwordEncoder) {
        return new ImageCodeAuthenticationSecurityConfig(loginFailureHandler, loginSuccessHandler, userDetailsService, passwordEncoder);
    }

    @Bean
    public ThirdQrAuthenticationSecurityConfig weChatQrAuthenticationSecurityConfig(
            LoginSuccessHandler loginSuccessHandler,
            LoginFailureHandler loginFailureHandler,
            TimingUserDetailService userDetailsService) {
        return new ThirdQrAuthenticationSecurityConfig(loginFailureHandler, loginSuccessHandler, userDetailsService);
    }
}
